@@ -27,7 +27,16 @@ Killbots::View::View( QGraphicsScene * scene, QWidget * parent )
 {
 	setMinimumSize( QSize( 300, 200 ) );
 	setFrameShape( QFrame::NoFrame );
-	setBackgroundRole( QPalette::Window );
+
+	// This makes the background of the widget transparent so that Oxygen's
+	// (or any other style's) window gradient is visible in unpainted areas of
+	// the scene.
+	QPalette p = palette();
+	QColor c = p.color( QPalette::Base );
+	c.setAlpha( 0 );
+	p.setColor( QPalette::Base, c );
+	setPalette( p );
+	setBackgroundRole( QPalette::Base );
 
 	setCacheMode( QGraphicsView::CacheBackground );
 	setViewportUpdateMode( QGraphicsView::SmartViewportUpdate );
