@@ -38,6 +38,7 @@
 #include "dealer.h"
 
 #include "gamestate.h"
+#include "messagebox.h"
 #include "renderer.h"
 #include "speeds.h"
 #include "version.h"
@@ -52,8 +53,6 @@
 #include <KMessageBox>
 #include <KRandom>
 #include <KSharedConfig>
-#include <KTemporaryFile>
-#include <KIO/NetAccess>
 
 #include <QtCore/QCoreApplication>
 #include <QtCore/QMutex>
@@ -62,9 +61,7 @@
 #include <QtCore/QThread>
 #include <QtCore/QTimer>
 #include <QtGui/QGraphicsSceneMouseEvent>
-#include <QtGui/QGraphicsView>
 #include <QtGui/QPainter>
-#include <QtGui/QStyleOptionGraphicsItem>
 #include <QtXml/QDomDocument>
 
 #include <cmath>
@@ -75,9 +72,26 @@
 namespace
 {
     const qreal wonBoxToSceneSizeRatio = 0.7;
-    const qreal textToWonBoxRatio = 0.8;
-    const int minimumWonBoxFontSize = 5;
-    const int maximumWonBoxFontSize = 36;
+
+    QList<KCard*> shuffled( const QList<KCard*> & cards, int seed )
+    {
+        Q_ASSERT( seed > 0 );
+
+        QList<KCard*> result = cards;
+        for ( int i = result.size(); i > 1; --i )
+        {
+            // We use the same pseudorandom number generation algorithm as Windows
+            // Freecell, so that game numbers are the same between the two applications.
+            // For more inforation, see
+            // http://support.microsoft.com/default.aspx?scid=kb;EN-US;Q28150
+            seed = 214013 * seed + 2531011;
+            int rand = ( seed >> 16 ) & 0x7fff;
+
+            result.swap( i - 1, rand % i );
+        }
+
+        return result;
+    }
 }
 
 
@@ -143,7 +157,7 @@ public:
 
     Solver * solver;
     SolverThread * solverThread;
-    QGraphicsPixmapItem * wonItem;
+    MessageBox * wonItem;
 
     QTimer * updateSolver;
     QTimer * demoTimer;
@@ -163,6 +177,9 @@ public:
     bool dropQueued;
     bool newCardsQueued;
 
+    QList<QPair<KCard*,KCardPile*> > multiStepMoves;
+    int multiStepDuration;
+
     QStack<GameState*> undoStack;
     GameState * currentState;
     QStack<GameState*> redoStack;
@@ -179,8 +196,10 @@ int DealerScene::moveCount() const
 }
 
 
-void DealerScene::saveGame(QDomDocument &doc)
+void DealerScene::saveGame( QIODevice * io )
 {
+    QDomDocument doc("kpat");
+
     QDomElement dealer = doc.createElement("dealer");
     doc.appendChild(dealer);
     dealer.setAttribute("id", gameId());
@@ -201,21 +220,28 @@ void DealerScene::saveGame(QDomDocument &doc)
         foreach(const KCard * c, p->cards() )
         {
             QDomElement card = doc.createElement("card");
-            card.setAttribute("suit", getSuit( c ));
-            card.setAttribute("value", getRank( c ));
+            card.setAttribute("suit", c->suit());
+            card.setAttribute("value", c->rank());
             card.setAttribute("faceup", c->isFaceUp());
             pile.appendChild(card);
         }
         dealer.appendChild(pile);
-     }
+    }
+
+    QTextStream stream( io );
+    stream.setCodec("UTF-8");
+    stream << doc.toString();
 
     d->gameWasJustSaved = true;
 }
 
-void DealerScene::openGame(QDomDocument &doc)
+void DealerScene::openGame( QIODevice * io )
 {
     resetInternals();
 
+    QDomDocument doc;
+    doc.setContent( io );
+
     QDomElement dealer = doc.documentElement();
 
     QString options = dealer.attribute("options");
@@ -261,9 +287,10 @@ void DealerScene::openGame(QDomDocument &doc)
                     QDomElement card = cardNodes.item(j).toElement();
                     int s = card.attribute("suit").toInt();
                     int r = card.attribute("value").toInt();
-
+                    
                     KCard * c = cards.take( ( s << 4 ) + r );
-                    Q_ASSERT( c );
+                    if (!c)
+                        continue;
 
                     c->setFaceUp(card.attribute("faceup").toInt());
                     p->add(c);
@@ -271,7 +298,7 @@ void DealerScene::openGame(QDomDocument &doc)
                 break;
             }
         }
-        p->layoutCards( 0 );
+        updatePileLayout( p, 0 );
     }
     setGameState( dealer.attribute("data") );
 
@@ -336,9 +363,10 @@ DealerScene::DealerScene()
     connect( d->dropTimer, SIGNAL(timeout()), this, SLOT(drop()) );
     d->dropTimer->setSingleShot( true );
 
-    d->wonItem  = new QGraphicsPixmapItem( 0, this );
+    d->wonItem = new MessageBox();
     d->wonItem->setZValue( 2000 );
     d->wonItem->hide();
+    addItem( d->wonItem );
 
     d->currentState = 0;
 
@@ -399,20 +427,22 @@ QList<PatPile*> DealerScene::patPiles() const
 }
 
 
-void DealerScene::moveCardsToPile( QList<KCard*> cards, KCardPile * pile, int duration )
+void DealerScene::cardsMoved( const QList<KCard*> & cards, KCardPile * oldPile, KCardPile * newPile )
 {
-    PatPile * newPile = dynamic_cast<PatPile*>( pile );
-    foreach ( KCard * c, cards )
+    PatPile * newPatPile = dynamic_cast<PatPile*>( newPile );
+    PatPile * oldPatPile = dynamic_cast<PatPile*>( oldPile );
+
+    if ( oldPatPile && oldPatPile->isFoundation() && newPatPile && !newPatPile->isFoundation() )
     {
-        PatPile * oldPile = dynamic_cast<PatPile*>( c->pile() );
-        if ( oldPile && oldPile->isFoundation() && !newPile->isFoundation() )
+        foreach ( KCard * c, cards )
             d->cardsNotToDrop.insert( c );
-        else
+    }
+    else
+    {
+        foreach ( KCard * c, cards )
             d->cardsNotToDrop.remove( c );
     }
 
-    KCardScene::moveCardsToPile( cards, pile, duration );
-
     if ( !d->dropInProgress && !d->dealInProgress )
     {
         d->gameStarted = true;
@@ -447,12 +477,9 @@ void DealerScene::startHint()
             toHighlight << mh.card();
     }
 
-    if ( !toHighlight.isEmpty() )
-    {
-        d->hintInProgress = true;
-        setHighlightedItems( toHighlight );
-        emit hintActive( true );
-    }
+    d->hintInProgress = !toHighlight.isEmpty();
+    setHighlightedItems( toHighlight );
+    emit hintActive( d->hintInProgress );
 }
 
 
@@ -472,7 +499,6 @@ bool DealerScene::isHintActive() const
     return d->hintInProgress;
 }
 
-
 QList<MoveHint> DealerScene::getSolverHints()
 {
     QList<MoveHint> hintList;
@@ -481,23 +507,16 @@ QList<MoveHint> DealerScene::getSolverHints()
         d->solverThread->abort();
 
     solver()->translate_layout();
-    solver()->patsolve( 1 );
+    bool debug = false;
+#if DEBUG_HINTS
+    debug = true;
+#endif
+    solver()->patsolve( 1, debug );
 
     foreach ( const MOVE & m, solver()->firstMoves )
     {
-
-#if DEBUG_HINTS
-        if ( m.totype == O_Type )
-            fprintf( stderr, "   move from %d out (at %d) Prio: %d\n", m.from,
-                     m.turn_index, m.pri );
-        else
-            fprintf( stderr, "   move from %d to %d (%d) Prio: %d\n", m.from, m.to,
-                     m.turn_index, m.pri );
-#endif
-
         MoveHint mh = solver()->translateMove( m );
-        if ( mh.isValid() )
-            hintList << mh;
+	hintList << mh;
     }
     return hintList;
 }
@@ -559,6 +578,11 @@ QList<MoveHint> DealerScene::getHints()
     return hintList;
 }
 
+static bool prioSort(const MoveHint &c1, const MoveHint &c2) 
+{
+  return c1.priority() < c2.priority();
+}
+
 
 MoveHint DealerScene::chooseHint()
 {
@@ -591,7 +615,7 @@ MoveHint DealerScene::chooseHint()
         qreal randomExp = qMin<qreal>( -log( 1 - qreal( KRandom::random() ) / RAND_MAX ) / 4, 1 );
         int randomIndex =  randomExp * ( hintList.size() - 1 );
 
-        qSort( hintList.begin(), hintList.end(), qGreater<MoveHint>() );
+	qSort(hintList.begin(), hintList.end(), prioSort);
         return hintList.at( randomIndex );
     }
 }
@@ -613,7 +637,7 @@ void DealerScene::startNew(int gameNumber)
 
     if ( isCardAnimationRunning() )
     {
-        QTimer::singleShot( 100, this, SLOT( startNew() ) );
+        QTimer::singleShot( 100, this, SLOT(startNew()) );
         return;
     }
 
@@ -621,15 +645,14 @@ void DealerScene::startNew(int gameNumber)
         d->solverThread->abort();
 
     resetInternals();
-    d->loadedMoveCount = 0;
-    d->gameWasJustSaved = false;
 
     emit updateMoves( 0 );
 
-    d->dealInProgress = true;
     foreach( KCardPile * p, piles() )
         p->clear();
-    restart();
+
+    d->dealInProgress = true;
+    restart( shuffled( deck()->cards(), d->gameNumber ) );
     d->dealInProgress = false;
 
     takeState();
@@ -652,9 +675,11 @@ void DealerScene::resetInternals()
     qDeleteAll( d->redoStack );
     d->redoStack.clear();
 
+    d->gameWasJustSaved = false;
     d->gameWasEverWinnable = false;
     d->toldAboutLostGame = false;
     d->toldAboutWonGame = false;
+    d->loadedMoveCount = 0;
 
     d->playerReceivedHelp = false;
 
@@ -701,6 +726,8 @@ void DealerScene::won()
     stopDemo();
     recordGameStatistics();
 
+    emit solverStateChanged( QString() );
+
     emit demoPossible( false );
     emit hintPossible( false );
     emit dropPossible( false );
@@ -746,12 +773,12 @@ void DealerScene::showWonMessage()
     d->wonItem->show();
 }
 
-void DealerScene::updateWonItem( bool force )
+void DealerScene::updateWonItem()
 {
     const qreal aspectRatio = Renderer::self()->aspectRatioOfElement("message_frame");
     int boxWidth;
     int boxHeight;
-    if (width() < aspectRatio * height())
+    if ( width() < aspectRatio * height() )
     {
         boxWidth = width() * wonBoxToSceneSizeRatio;
         boxHeight = boxWidth / aspectRatio;
@@ -761,46 +788,23 @@ void DealerScene::updateWonItem( bool force )
         boxHeight = height() * wonBoxToSceneSizeRatio;
         boxWidth = boxHeight * aspectRatio;
     }
+    d->wonItem->setSize( QSize( boxWidth, boxHeight ) );
 
-    // Only regenerate the pixmap if the new one is a significantly different size.
-    if ( force || qAbs( d->wonItem->boundingRect().width() - boxWidth ) > 20 )
-    {
-        QRect contentsRect = QRect( 0, 0, boxWidth, boxHeight );
-        QPixmap pix = Renderer::self()->renderElement( "message_frame", contentsRect.size() );
+    if ( d->playerReceivedHelp )
+        d->wonItem->setMessage( i18n( "Congratulations! We have won." ) );
+    else
+        d->wonItem->setMessage( i18n( "Congratulations! You have won." ) );
 
-        QString text;
-        if ( d->playerReceivedHelp )
-            text = i18n( "Congratulations! We have won." );
-        else
-            text = i18n( "Congratulations! You have won." );
-
-        QFont font;
-        int fontsize = maximumWonBoxFontSize;
-        font.setPointSize( fontsize );
-        int twidth = QFontMetrics( font ).width( text );
-        while ( twidth > boxWidth * textToWonBoxRatio && fontsize > minimumWonBoxFontSize )
-        {
-            fontsize--;
-            font.setPointSize( fontsize );
-            twidth = QFontMetrics( font ).width ( text );
-        }
-
-        QPainter p( &pix );
-        p.setFont( font );
-        p.setPen( Renderer::self()->colorOfElement("message_text_color") );
-        p.drawText( contentsRect, Qt::AlignCenter, text );
-        p.end();
-
-        d->wonItem->setPixmap( pix );
-    }
-
-    d->wonItem->setPos( QPointF( ( width() - d->wonItem->sceneBoundingRect().width() ) / 2,
-                                 ( height() - d->wonItem->sceneBoundingRect().height() ) / 2 ) + sceneRect().topLeft() );
+    d->wonItem->setPos( QPointF( (width() - boxWidth) / 2, (height() - boxHeight) / 2 )
+                        + sceneRect().topLeft() );
 }
 
 
 bool DealerScene::allowedToAdd( const KCardPile * pile, const QList<KCard*> & cards ) const
 {
+    if ( !pile->isEmpty() && !pile->topCard()->isFaceUp() )
+        return false;
+
     const PatPile * p = dynamic_cast<const PatPile*>( pile );
     return p && checkAdd( p, p->cards(), cards );
 }
@@ -809,11 +813,11 @@ bool DealerScene::allowedToAdd( const KCardPile * pile, const QList<KCard*> & ca
 bool DealerScene::allowedToRemove( const KCardPile * pile, const KCard * card ) const
 {
     const PatPile * p = dynamic_cast<const PatPile*>( pile );
-    if ( !p )
-        return false;
-
-    QList<KCard*> cards = p->topCardsDownTo( card );
-    return !cards.isEmpty() && checkRemove( p, cards );
+    QList<KCard*> cards = pile->topCardsDownTo( card );
+    return p
+           && card->isFaceUp()
+           && !cards.isEmpty()
+           && checkRemove( p, cards );
 }
 
 
@@ -856,7 +860,7 @@ void DealerScene::mousePressEvent( QGraphicsSceneMouseEvent * e )
     else if ( e->button() == Qt::RightButton
               && card
               && card->pile()
-              && card != card->pile()->top()
+              && card != card->pile()->topCard()
               && cardsBeingDragged().isEmpty()
               && !isCardAnimationRunning() )
     {
@@ -869,6 +873,8 @@ void DealerScene::mousePressEvent( QGraphicsSceneMouseEvent * e )
     else
     {
         KCardScene::mousePressEvent( e );
+        if ( !cardsBeingDragged().isEmpty() )
+            emit cardsPickedUp();
     }
 }
 
@@ -880,12 +886,15 @@ void DealerScene::mouseReleaseEvent( QGraphicsSceneMouseEvent * e )
     if ( e->button() == Qt::RightButton && d->peekedCard && d->peekedCard->pile() )
     {
         e->accept();
-        d->peekedCard->pile()->layoutCards( DURATION_FANCYSHOW );
+        updatePileLayout( d->peekedCard->pile(), DURATION_FANCYSHOW );
         d->peekedCard = 0;
     }
     else
     {
+        bool hadCards = !cardsBeingDragged().isEmpty();
         KCardScene::mouseReleaseEvent( e );
+        if ( hadCards && cardsBeingDragged().isEmpty() )
+            emit cardsPutDown();
     }
 }
 
@@ -903,7 +912,7 @@ bool DealerScene::tryAutomaticMove( KCard * card )
     if ( !isCardAnimationRunning()
          && card
          && card->pile()
-         && card == card->pile()->top()
+         && card == card->pile()->topCard()
          && card->isFaceUp()
          && allowedToRemove( card->pile(), card ) )
     {
@@ -1112,7 +1121,7 @@ void DealerScene::setState( GameState * state )
         foreach ( KCard * c, p->cards() )
             p->swapCards( p->indexOf( c ), cardIndices.value( c ) );
 
-        p->layoutCards(0);
+        updatePileLayout( p, 0 );
     }
 
     // restore game-specific information
@@ -1316,6 +1325,12 @@ void DealerScene::animationDone()
 {
     Q_ASSERT( !isCardAnimationRunning() );
 
+    if ( !d->multiStepMoves.isEmpty() )
+    {
+        continueMultiStepMove();
+        return;
+    }
+
     if ( d->takeStateQueued )
     {
         d->takeStateQueued = false;
@@ -1407,51 +1422,34 @@ void DealerScene::demo()
     MoveHint mh = chooseHint();
     if ( mh.isValid() )
     {
-        kDebug(mh.pile()->top()) << "Moving" << mh.card()->objectName()
-                                 << "from the" << mh.card()->pile()->objectName()
-                                 << "pile to the" << mh.pile()->objectName()
-                                 << "pile, putting it on top of" << mh.pile()->top()->objectName();
-        kDebug(!mh.pile()->top()) << "Moving" << mh.card()->objectName()
-                                  << "from the" << mh.card()->pile()->objectName()
-                                  << "pile to the" << mh.pile()->objectName()
-                                  << "pile, which is empty";
-        Q_ASSERT(mh.card()->pile() == 0
-                 || allowedToRemove(mh.card()->pile(), mh.card()));
+        KCard * card = mh.card();
+        Q_ASSERT( card );
+        KCardPile * sourcePile = mh.card()->pile();
+        Q_ASSERT( sourcePile );
+        Q_ASSERT( allowedToRemove( sourcePile, card ) );
+        PatPile * destPile = mh.pile();
+        Q_ASSERT( destPile );
+        Q_ASSERT( sourcePile != destPile );
+        QList<KCard*> cards = sourcePile->topCardsDownTo( card );
+        Q_ASSERT( allowedToAdd( destPile, cards ) );
 
-        QList<KCard*> empty;
-        QList<KCard*> cards = mh.card()->pile()->cards();
-        bool after = false;
-        for (QList<KCard*>::Iterator it = cards.begin(); it != cards.end(); ++it) {
-            if (*it == mh.card())
-                after = true;
-            if (after)
-                empty.append(*it);
+        if ( destPile->isEmpty() )
+        {
+            kDebug() << "Moving" << card->objectName()
+                     << "from the" << sourcePile->objectName()
+                     << "pile to the" << destPile->objectName()
+                     << "pile, which is empty";
         }
-
-        Q_ASSERT(!empty.isEmpty());
-
-        QMap<KCard*,QPointF> oldPositions;
-
-        foreach (KCard *c, empty) {
-            c->completeAnimation();
-            c->setFaceUp(true);
-            oldPositions.insert(c, c->pos());
-        }
-
-        Q_ASSERT(mh.card());
-        Q_ASSERT(mh.card()->pile());
-        Q_ASSERT(mh.pile());
-        Q_ASSERT(mh.card()->pile() != mh.pile());
-        Q_ASSERT(mh.pile()->isFoundation() || allowedToAdd(mh.pile(), empty));
-
-        moveCardsToPile( empty, mh.pile(), DURATION_MOVE );
-
-        foreach (KCard *c, empty) {
-            c->completeAnimation();
-            QPointF destPos = c->pos();
-            c->setPos(oldPositions.value(c));
-            c->animate(destPos, c->zValue(), 0, c->isFaceUp(), true, DURATION_DEMO);
+        else
+        {
+            kDebug() << "Moving" << card->objectName()
+                     << "from the" << sourcePile->objectName()
+                     << "pile to the" << destPile->objectName()
+                     << "pile, putting it on top of"
+                     << destPile->topCard()->objectName();
         }
+        
+        moveCardsToPile( cards, destPile, DURATION_DEMO );
     }
     else if ( !newCards() )
     {
@@ -1604,27 +1602,25 @@ int DealerScene::neededFutureMoves() const { return d->neededFutureMoves; }
 void DealerScene::setNeededFutureMoves( int i ) { d->neededFutureMoves = i; }
 
 
-QString DealerScene::save_it()
+void DealerScene::setDeckContents( int copies, const QList<KCardDeck::Suit> & suits )
 {
-    // If the game has been won, there's no current state to save.
-    if ( d->gameHasBeenWon )
-        return QString();
+    Q_ASSERT( copies >= 1 );
+    Q_ASSERT( !suits.isEmpty() );
 
-    KTemporaryFile file;
-    file.setAutoRemove(false);
-    file.open();
-    QDomDocument doc("kpat");
-    saveGame(doc);
-    QTextStream stream (&file);
-    stream << doc.toString();
-    stream.flush();
-    file.flush();
-    // .close will make fileName() return ""
-    QString filename = file.fileName();
-    file.close();
-    return filename;
+    // Note that the order in which the cards are created can not be changed
+    // without breaking the game numbering. For historical reasons, KPat
+    // generates card by rank and then by suit, rather than the more common
+    // suit then rank ordering.
+    QList<quint32> ids;
+    for ( int i = 0; i < copies; ++i )
+        foreach ( const KCardDeck::Rank & r, KCardDeck::standardRanks() )
+            foreach ( const KCardDeck::Suit & s, suits )
+                ids << KCardDeck::getId( s, r );
+
+    deck()->setDeckContents( ids );
 }
 
+
 QImage DealerScene::createDump() const
 {
     const QSize previewSize( 480, 320 );
@@ -1667,7 +1663,11 @@ QImage DealerScene::createDump() const
     return img2;
 }
 
-void DealerScene::mapOldId(int) {}
+
+void DealerScene::mapOldId( int id )
+{
+    Q_UNUSED( id );
+}
 
 
 int DealerScene::oldId() const
@@ -1676,6 +1676,30 @@ int DealerScene::oldId() const
 }
 
 
+QString DealerScene::getGameState() const
+{
+    return QString();
+}
+
+
+void DealerScene::setGameState( const QString & state )
+{
+    Q_UNUSED( state );
+}
+
+
+QString DealerScene::getGameOptions() const
+{
+    return QString();
+}
+
+
+void DealerScene::setGameOptions( const QString & options )
+{
+    Q_UNUSED( options );
+}
+
+
 bool DealerScene::allowedToStartNewGame()
 {
     // Check if the user is already running a game, and if she is,
@@ -1710,7 +1734,7 @@ void DealerScene::startDealAnimation()
     qreal speed = sqrt( width() * width() + height() * height() ) / ( DURATION_DEAL );
     foreach ( PatPile * p, patPiles() )
     {
-        p->layoutCards(0);
+        updatePileLayout( p, 0 );
         foreach ( KCard * c, p->cards() )
         {
             if ( !d->initDealPositions.contains( c ) )
@@ -1729,5 +1753,127 @@ void DealerScene::startDealAnimation()
 }
 
 
+void DealerScene::multiStepMove( const QList<KCard*> & cards,
+                                 KCardPile * pile,
+                                 const QList<KCardPile*> & freePiles,
+                                 const QList<KCardPile*> & freeCells,
+                                 int duration )
+{
+    Q_ASSERT( cards.size() == 1 || !freePiles.isEmpty() || !freeCells.isEmpty() );
+
+    d->multiStepMoves.clear();
+    d->multiStepDuration = duration;
+
+    multiStepSubMove( cards, pile, freePiles, freeCells );
+    continueMultiStepMove();
+}
+
+
+void DealerScene::multiStepSubMove( QList<KCard*> cards,
+                                    KCardPile * pile,
+                                    QList<KCardPile*> freePiles,
+                                    const QList<KCardPile*> & freeCells )
+{
+    // Note that cards and freePiles are passed by value, as we need to make a
+    // local copy anyway.
+
+    // Using n free cells, we can move a run of n+1 cards. If we want to move
+    // more than that, we have to recursively move some of our cards to one of
+    // the free piles temporarily.
+    const int freeCellsPlusOne = freeCells.size() + 1;
+    int cardsToSubMove = cards.size() - freeCellsPlusOne;
+
+    QList<QPair<KCardPile*,QList<KCard*> > > tempMoves;
+    while ( cardsToSubMove > 0 )
+    {
+        int tempMoveSize;
+        if ( cardsToSubMove <= freePiles.size() * freeCellsPlusOne )
+        {
+            // If the cards that have to be submoved can be spread across the
+            // the free piles without putting more than freeCellsPlusOne cards
+            // on each one, we do so. This means that none of our submoves will
+            // need further submoves, which keeps the total move count down. We
+            // Just to a simple rounding up integer division.
+            tempMoveSize = (cardsToSubMove + freePiles.size() - 1) / freePiles.size();
+        }
+        else
+        {
+            // Otherwise, we use the space optimal method that gets the cards
+            // moved using a minimal number of piles, but uses more submoves.
+            tempMoveSize = freeCellsPlusOne;
+            while ( tempMoveSize * 2 < cardsToSubMove )
+                tempMoveSize *= 2;
+        }
+
+        QList<KCard*> subCards;
+        for ( int i = 0; i < tempMoveSize; ++i )
+            subCards.prepend( cards.takeLast() );
+
+        Q_ASSERT( !freePiles.isEmpty() );
+        KCardPile * nextPile = freePiles.takeFirst();
+
+        tempMoves << qMakePair( nextPile, subCards );
+        multiStepSubMove( subCards, nextPile, freePiles, freeCells );
+
+        cardsToSubMove -= tempMoveSize;
+    }
+
+    // Move cards to free cells.
+    for ( int i = 0; i < cards.size() - 1; ++i )
+    {
+        KCard * c = cards.at( cards.size() - 1 - i );
+        d->multiStepMoves << qMakePair( c, freeCells[i] );
+    }
+
+    // Move bottom card to destination pile.
+    d->multiStepMoves << qMakePair( cards.first(), pile );
+
+    // Move cards from free cells to destination pile.
+    for ( int i = 1; i < cards.size(); ++i )
+        d->multiStepMoves << qMakePair( cards.at( i ), pile );
+
+    // If we just moved the bottomost card of the source pile, it must now be
+    // empty and we won't need it any more. So we return it to the list of free
+    // piles.
+    KCardPile * sourcePile = cards.first()->pile();
+    if ( sourcePile->at( 0 ) == cards.first() )
+        freePiles << sourcePile;
+
+    // If we had to do any submoves, we now move those cards from their
+    // temporary pile to the destination pile and free up their temporary pile.
+    while ( !tempMoves.isEmpty() )
+    {
+        QPair<KCardPile*, QList<KCard*> > m = tempMoves.takeLast();
+        multiStepSubMove( m.second, pile, freePiles, freeCells );
+        freePiles << m.first;
+    }
+}
+
+
+void DealerScene::continueMultiStepMove()
+{
+    Q_ASSERT( !d->multiStepMoves.isEmpty() );
+    Q_ASSERT( !isCardAnimationRunning() );
+
+    QPair<KCard*,KCardPile*> m = d->multiStepMoves.takeFirst();
+    KCard * card = m.first;
+    KCardPile * dest = m.second;
+    KCardPile * source = card->pile();
+
+    Q_ASSERT( card == source->topCard() );
+    Q_ASSERT( allowedToAdd( dest, QList<KCard*>() << card ) );
+
+    d->multiStepDuration = qMax<int>( d->multiStepDuration * 0.9, 50 );
+
+    dest->add( card );
+    card->raise();
+    updatePileLayout( dest, d->multiStepDuration );
+    updatePileLayout( source, d->multiStepDuration );
+
+    if ( d->multiStepMoves.isEmpty() )
+        takeState();
+}
+
+
 #include "dealer.moc"
 #include "moc_dealer.cpp"
