@@ -410,6 +410,9 @@ void View2DScene::init(const Game& game) {
 	connect(m_gameActions, SIGNAL(enterValue(int)), this, SLOT(enterValue(int)));
 	connect(m_gameActions, SIGNAL(markValue(int)), this, SLOT(flipMarkValue(int)));
 	connect(m_gameActions, SIGNAL(move(int,int)), this, SLOT(moveCursor(int,int)));
+	// Fix bug 188162 by ensuring that all markers, as well as cells, are
+	// updated and displayed after a Load action.
+	update(-1);
 }
 
 void View2DScene::setSceneSize(const QSize& size) {
@@ -420,7 +423,7 @@ void View2DScene::setSceneSize(const QSize& size) {
 	
 	int width = size.width() / (g->sizeX()+1);
 	int height = size.height() / (g->sizeY()+1);
-	int grid = ((width < height) ? width : height) / 2;
+	int grid = qMin(width, height) / 2;
 	int offsetX = size.width()/2 - g->sizeX()*grid;
 	int offsetY = size.height()/2 - g->sizeY()*grid;
 
